const createDOMPurify = require('dompurify');
const { JSDOM } = require('jsdom');

const window = new JSDOM('').window;
const DOMPurify = createDOMPurify(window);

// the belowfollowing is adapted from 
// https://danielpecos.com/2022/12/25/mastodon-as-comment-system-for-your-static-blog/
function escapeHtml(unsafe) {
    return unsafe
        .replace(/&/g, "&amp;")
        .replace(/</g, "&lt;")
        .replace(/>/g, "&gt;")
        .replace(/"/g, "&quot;")
        .replace(/'/g, "&#039;");
}

function fixEmojisInDisplayName({displayName, emojis}) {
    let fixedName = escapeHtml(displayName)
    emojis.forEach(emoji => {
        fixedName = fixedName.replace(
            `:${emoji.shortcode}:`, 
            `<img src="${escapeHtml(emoji.static_url)}" alt="Emoji ${emoji.shortcode}" height="20" width="20" />`
        )
    })
    return fixedName
}

module.exports = (data) => DOMPurify.sanitize(`<div class="mastodon-comment comment-depth-${Math.min(4, data.depth)}">
<div class="author">
    <div class="avatar">
    <img src="${escapeHtml(data.account.avatar)}" height=60 width=60 alt="">
    </div>
    <div class="details">
    <a class="name" href="${data.account.url}" rel="nofollow">${fixEmojisInDisplayName(data.account)}</a>
    <a class="user" href="${data.account.url}" rel="nofollow">${data.account.user}</a>
    </div>
    <a class="date" href="${data.url}" rel="nofollow"></a>
</div>
<div class="comment-content">${data.content}</div>
<div class="attachments">
    ${data.attachments.map(attachment => {
        if (attachment.type === 'image') {
            return `<a href="${attachment.url}" rel="nofollow"><img src="${attachment.preview_url}" alt="${attachment.description}" /></a>`;
        } else if (attachment.type === 'video') {
            return `<video controls><source src="${attachment.url}" type="${attachment.mime_type}"></video>`;
        } else if (attachment.type === 'gifv') {
            return `<video autoplay loop muted playsinline><source src="${attachment.url}" type="${attachment.mime_type}"></video>`;
        } else if (attachment.type === 'audio') {
            return `<audio controls><source src="${attachment.url}" type="${attachment.mime_type}"></audio>`;
        } else {
            return `<a href="${attachment.url}" rel="nofollow">${attachment.type}</a>`;
        }
    }).join('')}
</div>
<div class="status">
    <div class="replies ${data.counts.replies > 0 ? 'active' : ''}">
    <a href="${data.url}" rel="nofollow"><i class="fa fa-reply fa-fw"></i>${data.counts.replies || ''}</a>
    </div>
    <div class="reblogs ${data.counts.reblogs > 0 ? 'active' : ''}">
    <a href="${data.url}" rel="nofollow"><i class="fa fa-retweet fa-fw"></i>${data.counts.reblogs || ''}</a>
    </div>
    <div class="favourites ${data.counts.favourites > 0 ? 'active' : ''}">
    <a href="${data.url}" rel="nofollow"><i class="fa fa-star fa-fw"></i>${data.counts.favourites || ''}</a>
    </div>
</div>
</div>`);
